;+
;  zernike.pro - zernike modes
;       result = zernike(nx,ny,r,n,m)  - generate the zernike mode
;                                       on a 2-D (nx X ny) image
;       result = zernike(x,y,r,n,m)  - generate the zernike mode values
;                                      at positions given by vectors x and y
;          tab = ztable(nmax)  - generate a table of allowed n,m pairs
;
;   INPUTS
;         nx,ny - dimensions of image -or-
;          x,y  - arrays of x and y coordinates
;           r   - radius of aperture in pixels
;         n,m   - zernike indices, n-radial, m-angular
;   OUTPUT
;         res - returned as a complex pair
;              real part = z(n,m) cos(m theta)
;             imaginary part = z(n,m) sin(m theta)
;
;-

function zernikeslope,xa,ya,r,n,m,center=c
  x = xa
  y = ya
  if (n_elements(x) eq 1) then begin
    nx = x
    ny = y
    if (n_elements(c) eq 0) then begin
      c = [nx/2.,ny/2.]
    endif
    x = make_array(1,ny,/float,value=1) ## (real(indgen(nx)-c[0]))
    y = transpose(real(indgen(ny)-c[1])) ## make_array(nx,1,/float,value=1)
  endif
  u = double(sqrt(x^2 + y^2)/r)
  pr = 0*u
  prtheta = 0*u
  for s = 0,(n-m)/2 do begin
    if (n ne 2*s) then begin
      pr = pr + (-1)^s * (factorial(n-s) / factorial(s) ) / $
       (factorial((n+m)/2-s) * factorial((n-m)/2-s)) * $
            (n-2*s)*u^(n-2*s-1)
      prtheta = prtheta + (-1)^s * (factorial(n-s) / factorial(s) ) / $
       (factorial((n+m)/2-s) * factorial((n-m)/2-s)) * $
            u^(n-2*s-1)
    endif
  endfor
  th = atan(y,x)
  i = complex(0,1)
  retx = pr*cos(th)*(cos(m*th)+i*sin(m*th)) - $
        prtheta*sin(th)*(-m*sin(m*th)+i*m*cos(m*th))
  rety = pr*sin(th)*(cos(m*th)+i*sin(m*th)) + $
        prtheta*cos(th)*(-m*sin(m*th)+i*m*cos(m*th))
  return,[retx,rety]
end

function ztable,nmax
  tab = intarr(3,nmax+1)
  n = 0
  m = 0
  s = 0
  for k = 0, nmax do begin
    tab(0,k) = n
    tab(1,k) = m
    tab(2,k) = s
    if ((m ne 0) and (s eq 0)) then begin ; there is a sine part
      s = s+1
    endif else begin
      m = m+2
      s = 0
    endelse
    if (m gt n) then begin
      n = n+1
      if ((n/2)*2 eq n) then m = 0 else m = 1
      s = 0
    endif
  endfor
  return,tab
end

function zernike,xa,ya,r,n,m,center=c,l=l
	x = xa
	y = ya
	if (n_elements(x) eq 1) then begin
	nx = x
	ny = y
	if (n_elements(c) eq 0) then begin
	  c = [nx/2.,ny/2.]
	endif
	x = make_array(1,ny,/float,value=1) ## (real(indgen(nx)-c[0]))
	y = transpose(real(indgen(ny)-c[1])) ## make_array(nx,1,/float,value=1)
	endif
	u = double(sqrt(x^2 + y^2)/r)
	th = atan(y,x)
	p = 0*u
	for s = 0,(n-m)/2 do $
	p = p + (-1)^s * (factorial(n-s) / factorial(s) ) / $
	   (factorial((n+m)/2-s) * factorial((n-m)/2-s)) * $
	        u^(n-2*s)
	fuzz = 1e-8

	if (n_elements(l) eq 0) then begin
		i = complex(0,1)
		ret = p * ( cos(m*th) + i*sin(m*th) )
	endif else begin
		if (l eq 0) then ret = p*cos(m*th)
		if (l eq 1) then ret = p*sin(m*th)
	endelse
	return,ret
end
